<?php
namespace Financeiro\Logic\ContasReceber\LancamentoContasReceber;

use \Financeiro\Entities\Boletreceber;
use \Financeiro\Entities\Contareceber;
use Financeiro\Entities\Emprbanc;
use \Financeiro\Entities\Ratctreceber;
use \Financeiro\Entities\Ajctreceber;
use \Contabilidade\Entities\Contopectbl;
use \Fiscal\Entities\Notafiscal;
use Geral\Entities\Empresa;
use Geral\Repositories\Contcorr;
use \Geral\Util\NovoCodigo;
use \Geral\Util\DateUtil;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use \Financeiro\Logic\ContasReceber\LancamentoContasReceber\Exception;
use \Financeiro\Entities\Ajbolrec;
use \Contabilidade\Logic\Util;
use \Geral\Util\Ambiente;
use \Financeiro\Entities\Bancopref;
use Setforms2\Shared\Registry;


/**
 * Class OperacoesContaReceber
 * @package Financeiro\Logic\ContasReceber\LancamentoContasReceber
 */
class OperacoesContaReceber
{

    const TAMANHO_NRLANBOLREC = 10;
    const TAMANHO_NRLANCTONF = 10;
    const PREFIXO_NOVOCODIGO_BOLREC = 'BOLETRECEBER';
    const PREFIXO_NOVOCODIGO_NRLANCTONF = 'LANCNF/FILIAL';
    const IDBOLRECALTER_DEFAULT = '1';
    const IDFORMCADBREC_DEFAULT = '1';
    const STATUS_BOLETO_PENDENTE = 'A';
    const ID_BLOQUEADO = 'S';
    const ID_DOCFISCAL = 'N';
    const ID_DOCFINANC = 'S';
    const ID_ENTRSAIDOP = 'S';
    const ID_DOCCONTAB = 'S';
    const ID_DOCFISCENT = 'N';
    const ID_DOCFISCSAI = 'S';
    const ID_DOCESTOQUE = 'N';
    const ID_SISTEMA = 'REC';

    /** Gera Boleto e Conta a Receber */
    public static function geraBoletoContaReceber(
        $cdempresa, $cdtpctreceb, $dscontarec, $vrorigrec, $vratuarec, $dtatuavenrec,
        $nrorg, $dtorigvenrec, $idstatrec, $nrseqvenrec, $nrparcelarec, $cdfilial, $nrlanctonf,
        $cdfilibolrec, $cdclassfina, $dtmovfluxo, $idmovfluxo, $cdfimovflrec, $nrlanbolrec,
        $cdcliente, $qtdiasvenrec, $dtvenbolrec, $valormora, $valormulta, $percentualmora, $percentualmulta,
        $cdmotiparc, $p_nrparcelarec, $cdfilial_p
    )
    {

        try {
            $boletonovo = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilibolrec, $nrlanbolrec);
            if (!$boletonovo) {
                $boletonovo = new Boletreceber();
                $boletonovo->setCdfilial(\Geral\Util\Ambiente::getCdfilial());
                $boletonovo->setNrlanbolrec($nrlanbolrec);
            }

            /** @var $boletonovo BoletReceber */
            $boletonovo->setCdcliente($cdcliente);
            $boletonovo->setCdempresa($cdempresa);
            $boletonovo->setVrbolrec($vrorigrec);
            $boletonovo->setVrpemorbrec($percentualmora);
            $boletonovo->setVrpemulbrec($percentualmulta);
            $boletonovo->setVrmultabrec($valormulta);
            $boletonovo->setVrmordiabrec($valormora);
            $boletonovo->setVrfinalbrec($vrorigrec);
            $dtvenbolrec = self::formataData($dtvenbolrec);
            $boletonovo->setDtvenbolrec($dtvenbolrec);
            $boletonovo->setDtinclusao(DateUtil::getDataAtual());
            $boletonovo->setDtultatu(DateUtil::getDataAtual());
            $boletonovo->setCdoperultatu(Ambiente::getCdOperador());
            $boletonovo->setCdoperinclusao(Ambiente::getCdOperador());
            $boletonovo->setNrorg($nrorg);
            $boletonovo->setIdativo('S');

            self::setDadosBoletoBancoPref($boletonovo,$nrorg,$cdcliente,$cdempresa);

            $boletonovo->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            self::geraContaReceber(
                $cdempresa, $cdtpctreceb, $dscontarec, $vrorigrec, $vratuarec, $dtatuavenrec,
                $nrorg, $dtorigvenrec, $idstatrec, $nrseqvenrec, $nrparcelarec, $cdfilial,
                $nrlanctonf, $qtdiasvenrec, $cdfilibolrec, $cdclassfina, $dtmovfluxo, $idmovfluxo, $cdfimovflrec,
                $nrlanbolrec, $cdmotiparc, $p_nrparcelarec, $cdfilial_p
            );
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /** Busca dados parametrizaveis na bancopref e insere no boleto
     * @param $boletonovo
     * @param $nrorg
     * @param $cdcliente
     * @param $cdempresa
     */
    private function setDadosBoletoBancoPref(Boletreceber &$boletonovo,$nrorg,$cdcliente,$cdempresa){

        //Busca banco preferencial
        $rep = Bancopref::getRepository();
        /** @var $rep \Financeiro\Repositories\Bancopref   */
        $bancopref = $rep->getBancoPrefDadosBoleto($nrorg,$cdcliente,$cdempresa);

        if($bancopref['CDREDUCONT']){
            //Busca dados da conta corrente e emprbanc/
            $rep = Emprbanc::getRepository();
            /** @var $rep \Financeiro\Repositories\Emprbanc  */
            $emprbanc = $rep->getEmprBancDadosBoleto($nrorg,$bancopref['CDREDUCONT'],$cdempresa);

            if($bancopref['CDINS1COBRCL'])
                $boletonovo->setCdins1cobbol($bancopref['CDINS1COBRCL']);
            else{
                if($emprbanc['CDINS1COBR'])
                    $boletonovo->setCdins1cobbol($emprbanc['CDINS1COBR']);
            }
            if($bancopref['CDINS2COBRCL']){
                if($boletonovo->getCdins1cobbol()!= $bancopref['CDINS2COBRCL'])
                    $boletonovo->setCdins2cobbol($bancopref['CDINS2COBRCL']);
            }
            else{
                if($emprbanc['CDINS2COBR']){
                    if($boletonovo->getCdins1cobbol()!= $emprbanc['CDINS2COBR'])
                        $boletonovo->setCdins2cobbol($emprbanc['CDINS2COBR']);
                }
            }
            $boletonovo->setCdreducont($bancopref['CDREDUCONT']);
            $boletonovo->setCdcartcobbol($bancopref['CDCARTCOBR']);
            $boletonovo->setQtdiaprotbol($bancopref['QTDIAPROTEST']);

            if(! $emprbanc['IDIMPRBOLETO'])
                $boletonovo->setIdimprbolbol('B');
            else
                $boletonovo->setIdimprbolbol($emprbanc['IDIMPRBOLETO']);

            $boletonovo->setTxmensbolbol($bancopref['TXMENSBOLETO']);

            /** @var $empresa Empresa */
            $empresa = Empresa::findOneByNrorgAndCdempresa(Ambiente::getNrorg(), $cdempresa);
            if($empresa){
                if($empresa->getVrmultaempr()) {
                    $boletonovo->setVrpemulbrec($empresa->getVrmultaempr());
                    $boletonovo->setVrmultabrec($boletonovo->getVrbolrec() * $empresa->getVrmultaempr() / 100);
                }
                if($empresa->getVrmordiaempr()) {
                    $boletonovo->setVrmordiabrec($empresa->getVrmordiaempr());
                    $boletonovo->setVrmordiabrec($boletonovo->getVrbolrec() * $empresa->getVrmordiaempr() / 100);
                }
            }
        }
    }

    /** Geração de Boleto */
    public static function geraBoleto(
        $nrorg, $valortotal, $dtvencimento, $cdcliente, $cdempresa, $novocodigo, $percentualmora,
        $percentualmulta, $valormulta, $valormora
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $cdfilial = \Geral\Util\Ambiente::getCdfilial();
            /** @var $boletonovo BoletReceber */
            $boletonovo = new Boletreceber();
            $boletonovo->setCdfilial($cdfilial);
            $boletonovo->setNrlanbolrec($novocodigo);
            $boletonovo->setCdcliente($cdcliente);
            $boletonovo->setCdempresa($cdempresa);
            $boletonovo->setVrbolrec($valortotal);
            $boletonovo->setVrfinalbrec($valortotal);
            $boletonovo->setVrpemorbrec($percentualmora);
            $boletonovo->setVrpemulbrec($percentualmulta);
            $boletonovo->setVrmultabrec($valormulta);
            $boletonovo->setVrmordiabrec($valormora);
            $dtvencimento = self::formataData($dtvencimento);
            $boletonovo->setDtvenbolrec($dtvencimento);
            $boletonovo->setDtinclusao(DateUtil::getDataAtual());
            $boletonovo->setDtultatu(DateUtil::getDataAtual());
            $boletonovo->setCdoperultatu(Ambiente::getCdOperador());
            $boletonovo->setCdoperinclusao(Ambiente::getCdOperador());
            $boletonovo->setNrorg($nrorg);
            $boletonovo->setIdativo('S');

            self::setDadosBoletoBancoPref($boletonovo,$nrorg,$cdcliente,$cdempresa);

            $boletonovo->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

        public static function geraContaReceber(
        $cdempresa, $cdtpctreceb, $dscontarec, $vrorigrec, $vratuarec, $dtatuavenrec,
        $nrorg, $dtorigvenrec, $idstatrec, $nrseqvenrec, $nrparcelarec, $cdfilial,
        $nrlanctonf, $qtdiasvenrec, $cdfilibolrec, $cdclassfina, $dtmovfluxo, $idmovfluxo,
        $cdfimovflrec, $nrlanbolrec, $cdmotiparc, $p_nrparcelarec, $cdfilial_p
    )
    {
        try {
            $cdfilial = \Geral\Util\Ambiente::getCdfilial();
            if ($cdfilial_p != $cdfilial && $cdfilial_p != null)
                $cdfilial = $cdfilial_p;
            $contareceber = Contareceber::findOneByNrlanctonfAndNrorgAndNrseqvenrecAndCdfilialAndNrlanbolrecAndNrparcelarec(
                $nrlanctonf, $nrorg, $nrseqvenrec, $cdfilial, $nrlanbolrec, $p_nrparcelarec
            );
            if (!$contareceber) {
                $contareceber = new Contareceber();
                $contareceber->setQtdiasvenrec(0);
            }
            $contareceber->setCdempresa($cdempresa);
            $contareceber->setNrlanctonf($nrlanctonf);
            $contareceber->setCdtpctreceb($cdtpctreceb);
            $contareceber->setDscontarec($dscontarec);
            $contareceber->setVrorigrec($vrorigrec);
            $contareceber->setVratuarec($vratuarec);
            $dtatuavenrec = self::formataData($dtatuavenrec);
            $contareceber->setDtatuavenrec($dtatuavenrec);
            $contareceber->setNrorg($nrorg);
            $dtorigvenrec = self::formataData($dtorigvenrec);
            $contareceber->setDtorigvenrec($dtorigvenrec);
            $contareceber->setIdstatrec($idstatrec);
            $contareceber->setNrseqvenrec($nrseqvenrec);
            $contareceber->setNrparcelarec($nrparcelarec);
            $contareceber->setNrlanbolrec($nrlanbolrec);
            $contareceber->setCdfilial($cdfilial);
            $contareceber->setCdfilibolrec($cdfilibolrec);
            $contareceber->setCdclassfina($cdclassfina);
            $contareceber->setDtmovfluxo($dtatuavenrec);
            $contareceber->setIdmovfluxo($idmovfluxo);
            $contareceber->setCdfimovflrec($cdfimovflrec);
            $contareceber->setCdmotiparc($cdmotiparc);
            $contareceber->setIdativo('S');
            $contareceber->setQtdiasvenrec(0);
            $contareceber->setDtinclusao(DateUtil::getDataAtual());
            $contareceber->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function geraRateio(
        $cdcentcust, $vrratccctrec, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec,
        $nrparcelarec, $cdfiratctrec
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            $rateioreceber = Ratctreceber::findOneByNrorgAndNrlanctonfAndCdcentcustAndNrparcelarecAndNrseqvenrec(
                $nrorg, $nrlanctonf, $cdcentcust, $nrparcelarec, $nrseqvenrec
            );
            if (!$rateioreceber) {
                $rateioreceber = new Ratctreceber();
            }
            $rateioreceber->setCdcentcust($cdcentcust);
            $rateioreceber->setVrratccctrec($vrratccctrec);
            $rateioreceber->setNrorg($nrorg);
            $rateioreceber->setCdfilial($cdfilial);
            $rateioreceber->setNrlanctonf($nrlanctonf);
            $rateioreceber->setNrseqvenrec($nrseqvenrec);
            $rateioreceber->setNrparcelarec($nrparcelarec);
            $rateioreceber->setCdfiratctrec($cdfiratctrec);
            $now = self::formataData(now());
            $rateioreceber->setDtinclusao($now);
            $rateioreceber->persist();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function updateRateio(
        $cdcentcust, $vrratccctrec, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec,
        $nrparcelarec, $cdfiratctrec
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $rat = Ratctreceber::findOneByNrorgAndNrlanctonfAndCdfilialAndNrseqvenrecAndNrparcelarecAndCdfiratctrecAndCdcentcust(
                $nrorg, $nrlanctonf,
                $cdfilial, $nrseqvenrec, $nrparcelarec, $cdfiratctrec,
                $cdcentcust
            );
            $rat->setVrratccctrec($vrratccctrec);

            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function atualizaFluxodeCaixa(
        $cdempresa, $cdfilial, $cdclassFina, $dtMovFluxo, $valor, $idtipomov, $idOperacao
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            FluxodeCaixa::atualizaFluxo(
                $cdempresa, $cdfilial, $cdclassFina, $dtMovFluxo, $valor, $idtipomov, $idOperacao
            );

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function updateBoletoeConta(
        $nrorg, $cdfilial, $nrlanbolrec, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $valor, $cdfilibolrec
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var $boleto Boletreceber*/
            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilibolrec, $nrlanbolrec);
            $boleto->setVrbolrec($valor);
            $boleto->setVrfinalbrec($valor);
            $boleto->persist();

            /** @var $conta Contareceber */
            $conta = Contareceber::findOneByNrorgAndCdfilialAndNrlanctonfAndNrparcelarecAndNrseqvenrec(
                $nrorg, $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec
            );
            $conta->setVratuarec($valor);
            $conta->persist();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function updateProrrogacao(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfilibolrec, $nrlanbolrec, $vencimento, $cdmotipror
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilibolrec, $nrlanbolrec);
            $vencimento = self::formataData($vencimento);
            $boleto->setDtvenbolrec($vencimento);

            /** @var $contareceber Contareceber */
            $contareceber = Contareceber::findOneByNrorgAndCdfilialAndNrlanctonfAndNrparcelarecAndNrseqvenrec(
                $nrorg, $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec
            );
            $contareceber->setDtatuavenrec($vencimento);
            $contareceber->setCdmotipror($cdmotipror);

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function validaBoletoReal($cdfilibolrec, $nrlanbolrec)
    {
        try {
            $contareceber = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg($cdfilibolrec, $nrlanbolrec, Ambiente::getNrorg());
            if ($contareceber)
                    {
                        Exception\ExcecoesContasReceber::boleto_real();
                    }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaAjuste($cdfilial, $nrlanctonf)
    {
        try {
            $ajuste = Ajctreceber::findOneByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, Ambiente::getNrorg());
            if ($ajuste)
                Exception\ExcecoesContasReceber::conta_possui_ajuste();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaStatusConta($idstatrec)
    {
        try {
            if ($idstatrec != 'A') {
                Exception\ExcecoesContasReceber::conta_baixada();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdempresa
     * @param $data
     * @throws \Exception
     */
    public static function validaLancamento($cdempresa, $data, $nrorg)
    {
        try {
            /** @var $ajuste \Contabilidade\Repositories\Contopectbl */
            $ajuste = Contopectbl::getRepository();
            if($ajuste){
                    $data = $ajuste->vdPeriodoLancamento($cdempresa, $data, $nrorg);
                if ($data == null)
                    Exception\ExcecoesContasReceber::lancamento_fora_periodo();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrlanbolrec
     * @param $nrorg
     * @param $cdfilial
     * @throws \Exception
     */
    public static function validaBoletoEmitido($nrlanbolrec, $nrorg, $cdfilial)
    {
        try {
            $boleto = Boletreceber::findOneByNrlanbolrecAndNrorgAndCdfilial($nrlanbolrec, $nrorg, $cdfilial);
            if ($boleto->getIdstatbrec() == 'E')
                Exception\ExcecoesContasReceber::validaboletoemitido();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaBoletoCnab($nrlanbolrec, $cdfilial)
    {
        try {
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, Ambiente::getNrorg());
            if ($boleto->getNrcnabrecrem())
                throw new \Exception(tk_get_message_by_name('msg_pbi00015_validaboletocnab'));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaBoletoAssociado($nrorg, $nrlanbolrec, $cdfilial)
    {
        try {
            $boleto = Contareceber::findByNrorgAndNrlanbolrecAndCdfilial($nrorg, $nrlanbolrec, $cdfilial);
            if (count($boleto) > 1)
                Exception\ExcecoesContasReceber::boleto_real();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaNotaSistema($nrlanctonf, $cdfilial)
    {
        try {
            $nota = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, Ambiente::getNrorg());
            if ($nota->getIdsistema() != 'REC')
                throw new \Exception('Apenas lançamentos no módulo de contas a receber podem ser excluídos.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaDataEmissao($dtemissao, $dtentrsaid)
    {
        try {
            if (DateUtil::comparaDatas(self::formataData($dtemissao), self::formataData($dtentrsaid), 2)) {
                Exception\ExcecoesContasReceber::msg_data_emissao();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaDataVencimento($dtemissao, $dtatuavenrec)
    {
        try {
            if (DateUtil::comparaDatas(self::formataData($dtemissao), self::formataData($dtatuavenrec), 2)) {
                Exception\ExcecoesContasReceber::msg_data_vencimento();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function verificaAjusteBoleto($nrlanbolrec, $cdfilial)
    {
        try {
            $boleto = Ajbolrec::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, Ambiente::getNrorg());
            if ($boleto) {
                throw new \Exception(tk_get_message_by_name('msg_pbi00015_verificaajusteboleto'));
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    public static function contabiliza_conta_receber(
        $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $dtajusterec, $cdajuctrec, $idtipo
    )
    {
        try {
            //Inicia Transação
            $nrlancctb = '';
            $procedure = new Procedure('CTB_REC_LANC');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPARCELAREC', Param::PARAM_INPUT, $nrparcelarec, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQVENREC', Param::PARAM_INPUT, $nrseqvenrec, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTAJUSTEREC', Param::PARAM_INPUT, $dtajusterec, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDAJUCTREC', Param::PARAM_INPUT, $cdajuctrec, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTIPO', Param::PARAM_INPUT, $idtipo, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, $nrlancctb, Param::PARAM_TYPE_STR, 12));
            $nrlancctb = $procedure->call();
            return $nrlancctb['P_NRLANCCTB'];


        } catch (\Exception $e) {
            throw $e;
        }

    }

}